# Script 3 - Differential gene expression analysis

# Load required libraries
library(tidyr) 
suppressMessages({
  library("sleuth")
})
library(ggplot2)
library(ggrepel) # Used for text labels in MA and volcano plots
library(data.table)
library(dplyr)
library(formattable)

# Generate a design table based on the outputs of Kallisto and 
# the contents of design_table.txt which specifies which samples are LTBI
# and which samples are active infection
sample_id <- dir(file.path("results"))
kallisto_result_paths <- file.path(".", "results", sample_id)
kallisto_result_paths <- kallisto_result_paths[order(kallisto_result_paths)]
design_table <- read.table(file.path("design_table.txt"), header = TRUE, 
                           stringsAsFactors=FALSE)
design_table <- design_table[order(design_table$sample),]
design_table <- dplyr::mutate(design_table, path = kallisto_result_paths)
design_table <- design_table[order(design_table$condition),]

# May need this for biomaRt function to work
new_config <- httr::config(ssl_verifypeer = FALSE)
httr::set_config(new_config, override = FALSE)

# For gene annotation (relate transcripts to genes)
mart <- biomaRt::useMart(biomart = "ENSEMBL_MART_ENSEMBL",
                         dataset = "hsapiens_gene_ensembl",
                         host = 'ensembl.org')
t2g <- biomaRt::getBM(attributes = c("ensembl_transcript_id", "ensembl_gene_id",
                                     "external_gene_name"), mart = mart)
t2g <- dplyr::rename(t2g, target_id = ensembl_transcript_id,
                     ens_gene = ensembl_gene_id, ext_gene = external_gene_name)

# LRT for gene aggregated p-values
so_lrt <- sleuth_prep(design_table, 
                      extra_bootstrap_summary = TRUE, 
                      target_mapping = t2g, 
                      aggregation_column = 'ens_gene')
so_lrt <- sleuth_fit(so_lrt, ~condition, 'full')
so_lrt <- sleuth_fit(so_lrt, ~1, 'reduced')
so_lrt <- sleuth_lrt(so_lrt, 'reduced', 'full')
sleuth_table_lrt <- sleuth_results(so_lrt, 
                                   'reduced:full', 
                                   'lrt', 
                                   show_all = FALSE)
sleuth_significant_lrt <- dplyr::filter(sleuth_table_lrt, qval <= 0.05)

# Wald test for beta values (which are analogous to log 2 fold change values)
so_wt <- sleuth_prep(design_table, 
                     extra_bootstrap_summary = TRUE, 
                     target_mapping = t2g, 
                     aggregation_column = 'ens_gene', 
                     gene_mode = TRUE)
so_wt <- sleuth_fit(so_wt, ~condition, 'full')
so_wt <- sleuth_fit(so_wt, ~1, 'reduced')
so_wt <- sleuth_wt(so_wt, 
                   which_beta="conditionexperimental", 
                   which_model="full")
sleuth_table_wt <- sleuth_results(so_wt, 
                                  'conditionexperimental', 
                                  'wt', 
                                  show_all=FALSE, 
                                  gene_mode = TRUE)

# Merge beta values (and mean_obs for MA plot) into first table with aggregated 
# p-values
sleuth_table_all <- merge(x = sleuth_table_lrt, 
                          y = sleuth_table_wt[c("target_id", "b", "mean_obs")], 
                          by = "target_id", 
                          all.x = TRUE )

# PCA plot to show sample clustering
plot_pca(so_lrt, 
         color_by = c('condition')) +
  scale_color_discrete(name = "Condition", 
                       labels = c("LTBI", "Active Infection"))

# Formatted table with significant genes sorted by increasing q value
formatted_results <- dplyr::filter(sleuth_table_all, 
                                   qval <= 0.05, abs(b) > 1)[
                                      c("ext_gene", "qval", "b")]

names(formatted_results) <- c("Gene", "q", "beta")
formatted_results <- formatted_results[order(formatted_results$q, 
                                             decreasing= F),]
row.names(formatted_results) <- NULL # Get rid of rownames so they are not 
                                     # present in formatted table

# There will be a few duplicate rows for some reason - remove them
formatted_results <- distinct(formatted_results, formatted_results$Gene, 
                              .keep_all = TRUE)
formatted_results[4] <- NULL # Get rid of additional column
formatted_results <- formattable(formatted_results)
formatted_results

# MA plot
ggplot(sleuth_table_all, 
       aes(x=mean_obs, y=b, color=ifelse(b > 1.00 & qval < 0.05, "red", 
                                         ifelse(b < -1.00 & qval < 0.05, "blue", 
                                                "black")
       ))) +
  geom_point() +
  scale_color_manual(name = "Differential Expression", 
                     values=c("black", "blue", "red"), 
                     labels = c("Not DEG", 
                                "Downregulated (B < -1)",
                                "Upregulated (B > 1)")) +
  geom_text_repel(data = subset(
      head(sleuth_table_all[order(sleuth_table_all$qval, decreasing= F),], 
           n = 100), 
      abs(b)>1.1 & qval < 0.05), 
      aes(label=ext_gene), 
      colour = "purple",
      force = 9, 
      box.padding = .25) +
  xlab("Mean Normalized Abundance") +
  ylab("Beta Value (Analogous to Log2 Fold-Change)") + 
  theme(legend.position = "bottom",
        legend.title=element_text(size=9),
        legend.text=element_text(size=9))

# Volcano plot
ggplot(sleuth_table_all, 
       aes(x=b, y=-log10(qval), 
           color=ifelse(b > 1.00 & qval < 0.05, "red", 
                        ifelse(b < -1.00 & qval < 0.05, "blue", "black")))) + 
  geom_point() +
  scale_color_manual(name = "Differential Expression", 
                     values=c("black", "blue", "red"), 
                     labels = c("Not DEG", 
                                "Downregulated (B < -1)",
                                "Upregulated (B > 1)")) +
  ylim(0,15) +
  geom_text_repel(data = subset(
    head(sleuth_table_all[order(sleuth_table_all$qval, decreasing= F),], 
         n = 100), 
    abs(b)>1.1 & qval < 0.05), 
    aes(label=ext_gene), 
    colour = "purple",
    force = 9, 
    box.padding = .25) +
  xlab("Beta Value (Analogous to Log2 Fold-Change)") +
  ylab("-log10(q)") + 
  theme(legend.position = "bottom",
        legend.title=element_text(size=9),
        legend.text=element_text(size=9))
